#include<pic.h>
#include<pic1687x.h>

#define INVALID 100		// value of an invalid keypress

#define	ZERO		0x7e	// BCD to 7 segment display codes
#define	ONE		0x30
#define	TWO		0x6d
#define	THREE		0x79
#define	FOUR		0x33
#define	FIVE		0x5b
#define	SIX		0x5f
#define	SEVEN		0x70
#define	EIGHT		0x7f
#define	NINE		0x7b

__CONFIG(0x3f72);		// Disable Watch dog timer, no LVP, use high speed crystal (4MHz)

unsigned char key_pressed=INVALID, digit1;
unsigned char kb_flag=0;	
unsigned char old_key=INVALID;
unsigned char wait_for_release=0;
unsigned char key_pressed2 = INVALID;

unsigned char disp[4],disp_pos=0;
unsigned char temperature[2];

void interrupt isr(void);
unsigned char bcd2seg(int);

main()
{	
int tempr, tempr2=0;			//measured temperature &  entered temperature
float temp_const = .48828;		//constant to convert the read in analog value into temeprature in Centigrade

TRISB = 0b00000000;			//use PORTB for 7 segments segment 'a' is RB6 'g' is RB0
TRISD = 0b00000000;			// use PORTD for display select
TRISC = 0b01110000;			//use PORTC for keypad input
TRISE = 0b00000000;			// use RE0 for heater
T1CON = 0b000000001;
TMR1IF = 0;				//reset timer1 flag
TMR1IE = 1;				//enable timer1 interrupts
T0IE = 1;				//enable timer0 interrupts
OPTION = 0b11010000;			//no prescaler for timer0
PEIE = 1;				//enable peripheral interrupts
PORTC=0;
ADCON1 = 0b10001110;			//use RA0 as analog input & rest as digital
ADCON0 = 0b00000001;
TRISA =0xff;        
GIE = 1;				//enable interrupts
TMR0=0;
TMR1H = 0;				//clear timers
TMR1L = 0;
PORTB = 0;				//clear ports
PORTD = 0;
ADRESL	=0; 	// low byte of ADC output
ADRESH	=0;     // high byte of ADC output


disp[2]=bcd2seg(0);	//display "00" as the entered temperature in the begining
disp[3]=bcd2seg(0);


while(1)
{

if(kb_flag==1) {			//kb_flag=1 means there is a key pressed
	if(key_pressed == 12) {		//if this is # then take next two digits as temperature settings
		disp[2]=bcd2seg(0);	//clear user input display to 00
		disp[3]=bcd2seg(0);	
		kb_flag=0;		//enable key inputs

		while(kb_flag==0);			//Error checking
		if(key_pressed>9){			//Invalid key
			disp[2]=~0x4F;			//Display "E"
			disp[3]=~0x05;			//Display "r"
			kb_flag=0;
			goto there;
		}
		else{					//1st digit of temperature
			digit1=key_pressed;		//save the first key pressed
			disp[2]=bcd2seg(key_pressed);	//display it
		}
		kb_flag = 0;
		
		while(kb_flag==0);
		
		if(key_pressed>9){			//Invalid key
			disp[2]=~0x4F;			//Display "E"
			disp[3]=~0x06;			//Display "r"
			kb_flag=0;
			goto there;
		}		
		else{					//2nd digit of temperature
			temperature[0]=digit1;		
			temperature[1]=key_pressed;
			disp[3]=bcd2seg(key_pressed);	//display it
		}
		kb_flag =0;
		tempr2=(int)(temperature[0]*10+temperature[1]);		//convert the digits into decimal format

	}
	else kb_flag=0;
}

there:

ADGO=1;			//start analog conversion to measure current temperature
while(ADGO==1);		//wait till conversion is complete
tempr = (int)(temp_const*ADRESL);	// convert the value to temperature in centigrade
disp[0]=bcd2seg(tempr/10);		//display each digit
disp[1]= bcd2seg(tempr%10);		//of the temperature

if(tempr2 <= tempr-1)		//provide a hysterisis of 2 degrees
	RE0=0;			//turn off heater
if(tempr2 >= tempr+1)
	RE0=1;			//turn on heater

}

}


unsigned char bcd2seg(int key)			// convert BCD to 7 segment code
{
switch(key)
	{
	case 0:return(~(ZERO));
	case 1:return(~(ONE));
	case 2:return(~(TWO));
	case 3:return(~(THREE));
	case 4:return(~(FOUR));
	case 5:return(~(FIVE));
	case 6:return(~(SIX));
	case 7:return(~(SEVEN));
	case 8:return(~(EIGHT));
	case 9:return(~(NINE));
	default:return(0);
	}
}



void interrupt isr(void)
{
if(TMR1IF)		//interrupt for keyboard
{			//this routine gets a valid key and stores it in key_pressed. Also sets kb_flag
TMR1IF=0;

old_key=key_pressed2;

if(kb_flag==0 && wait_for_release == 0)
{
	key_pressed2 = INVALID;
	RC0=1;RC1=0;RC2=0;RC3=0;
	if(RC6 == 1)
	key_pressed2 = 1;
        else if(RC5==1)
	key_pressed2 =2 ;
        else if(RC4==1)
	key_pressed2 =3 ;

	RC0=0;RC1=1;RC2=0;RC3=0;
	if(RC6 == 1)
	key_pressed2 = 4;
        else if(RC5==1)
	key_pressed2 =5 ;
        else if(RC4==1)
	key_pressed2 =6 ;

	RC0=0;RC1=0;RC2=1;RC3=0;
	if(RC6 == 1)
	key_pressed2 = 7;
        else if(RC5==1)
	key_pressed2 =8 ;
        else if(RC4==1)
	key_pressed2 =9 ;

	RC0=0;RC1=0;RC2=0;RC3=1;
	if(RC6 == 1)
	key_pressed2 = 11;
        else if(RC5==1)
	key_pressed2 = 0 ;
        else if(RC4==1)
	key_pressed2 =12 ;

	RC0=0;RC1=0;RC2=0;RC3=0;
	
	if(old_key == key_pressed2 && key_pressed2!=INVALID)
	wait_for_release = 1;
}
else if(wait_for_release == 1)
{

	RC0=1;RC1=1;RC2=1;RC3=1;
	if(RC4==0 && RC5 == 0 && RC6 == 0)
	{
	key_pressed = key_pressed2;
	kb_flag = 1;
	wait_for_release = 0;


	}
}
}


if(T0IF)		//interrupt for display
{			// handles 4 digit multiplexed display Leftmost display CA is connected to RD0
  T0IF = 0;

  PORTD = 0 ;  // To ensure nothings happens while switching display
  switch(disp_pos)
  {
   case 1: disp_pos = 2;PORTB  = disp[1]; break; 
   case 2: disp_pos = 4;PORTB  = disp[2]; break;
   case 4: disp_pos = 8;PORTB  = disp[3]; break;
   case 8: disp_pos = 1;PORTB  = disp[0]; break;
   default: disp_pos = 1;
  }

  PORTD = disp_pos;
}

}


